@@ -1,22 +1,22 @@
-/*
+/* 
  * Seahorse
- *
- * Copyright (C) 2005 Nate Nielsen
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * 
+ * Copyright (C) 2005 Nate Nielsen 
+ * 
+ * This program is free software; you can redistribute it and/or modify 
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *  
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *  
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.  
  */
 
 #include "cryptui-keyset.h"
@@ -176,16 +176,15 @@ lookup_key_property (CryptUIKeyset *keyset, const gchar *key, const gchar *prop,
     }
     
     value = g_new0 (GValue, 1);
-    if (!dbus_g_proxy_call (keyset->priv->remote_service, "GetKeyField", &error,
+    if (!dbus_g_proxy_call (keyset->priv->remote_keyset, "GetKeyField", &error,
                             G_TYPE_STRING, key, G_TYPE_STRING, prop, G_TYPE_INVALID, 
-                            G_TYPE_VALUE, value, G_TYPE_INVALID)) {
+                            G_TYPE_BOOLEAN, allocated, G_TYPE_VALUE, value, G_TYPE_INVALID)) {
         g_warning ("dbus call to get '%s' failed: %s", prop, error ? error->message : "");
         g_clear_error (&error);
         g_free (value);
         return NULL;
     }
 
-    *allocated = TRUE;
     return value;
 }
 
@@ -237,9 +236,9 @@ cryptui_keyset_constructor (GType type, guint n_props, GObjectConstructParam* pr
     path = g_strdup_printf("/org/gnome/seahorse/keys/%s", keyset->priv->keytype);
 
     keyset->priv->remote_keyset = dbus_g_proxy_new_for_name (bus,
-                    "org.gnome.seahorse.KeyService", path, "org.gnome.seahorse.Keys");
+                    "org.gnome.seahorse", path, "org.gnome.seahorse.Keys");
     keyset->priv->remote_service = dbus_g_proxy_new_for_name (bus,
-                    "org.gnome.seahorse.KeyService", "/org/gnome/seahorse/keys", "org.gnome.seahorse.KeyService");
+                    "org.gnome.seahorse", "/org/gnome/seahorse/keys", "org.gnome.seahorse.KeyService");
     
     g_free (path);
             
@@ -381,6 +380,12 @@ cryptui_keyset_new (const gchar *keytype)
     return g_object_new (CRYPTUI_TYPE_KEYSET, "keytype", keytype, NULL);
 }
 
+const gchar*
+cryptui_keyset_get_keytype (CryptUIKeyset *keyset)
+{
+    return keyset->priv->keytype;
+}
+
 GList*             
 cryptui_keyset_get_keys (CryptUIKeyset *keyset)
 {
@@ -470,7 +475,7 @@ cryptui_keyset_cache_key (CryptUIKeyset *keyset, const gchar *key)
     if (props)
         return;
 
-    if (!dbus_g_proxy_call (keyset->priv->remote_service, "GetKeyFields", &error,
+    if (!dbus_g_proxy_call (keyset->priv->remote_keyset, "GetKeyFields", &error,
                             G_TYPE_STRING, key, G_TYPE_STRV, cached_key_props, G_TYPE_INVALID, 
                             TYPE_G_STRING_VALUE_HASHTABLE, &props, G_TYPE_INVALID)) {
         g_warning ("dbus call to cache key failed: %s", error ? error->message : "");
@@ -548,3 +553,41 @@ cryptui_keyset_key_flags (CryptUIKeyset *keyset, const gchar *key)
 {
     return cryptui_keyset_key_get_uint (keyset, key, "flags");
 }
+
+gchar*
+cryptui_keyset_key_raw_keyid (CryptUIKeyset *keyset, const gchar *key)
+{
+    return cryptui_keyset_key_get_string (keyset, key, "key-id");
+}
+
+gchar**
+cryptui_keyset_keys_raw_keyids (CryptUIKeyset *keyset, const gchar **keys)
+{
+    guint nkeys;
+    const gchar **k;
+    gchar **ids;
+    int i;
+    
+    /* TODO: This could get inefficient */
+    
+    for (k = keys, nkeys = 0; *k; k++)
+        nkeys++;
+    ids = g_new0 (gchar*, nkeys + 1);
+    for (k = keys, i = 0; *k; k++, i++)
+        ids[i] = cryptui_keyset_key_get_string (keyset, *k, "key-id");
+    
+    return ids;
+}
+
+/* -----------------------------------------------------------------------------
+ * INTERNAL
+ */
+
+const gchar*
+_cryptui_keyset_get_internal_keyid (CryptUIKeyset *keyset, const gchar *keyid)
+{
+    gpointer orig_key, value;
+    if (!g_hash_table_lookup_extended (keyset->priv->keys, keyid, &orig_key, &value))
+        return NULL;
+    return (const gchar*)orig_key;
+}
