@@ -57,16 +57,19 @@
 typedef struct _SeahorseContext SeahorseContext;
 typedef struct _SeahorseContextClass SeahorseContextClass;
 typedef struct _SeahorseContextPrivate SeahorseContextPrivate;
-	
+
 struct _SeahorseContext {
-	GtkObject		        parent;
-	
-	/*< private >*/
-	SeahorseContextPrivate	*pv;
+    GtkObject               parent;
+    
+    /*< public >*/
+    gboolean                is_daemon;
+    
+    /*< private >*/
+    SeahorseContextPrivate  *pv;
 };
 
 struct _SeahorseContextClass {
-	GtkObjectClass		parent_class;
+    GtkObjectClass parent_class;
     
     /* signals --------------------------------------------------------- */
     
@@ -80,17 +83,24 @@ struct _SeahorseContextClass {
     void (*changed) (SeahorseContext *sctx, SeahorseKey *skey, SeahorseKeyChange change);
 };
 
+enum SeahorseContextType {
+    SEAHORSE_CONTEXT_APP = 1,
+    SEAHORSE_CONTEXT_DAEMON = 2,
+};
+
 #define             SCTX_APP()                          (seahorse_context_app ())
 
 GType               seahorse_context_get_type           (void);
 
 SeahorseContext*    seahorse_context_app                (void);
 
-SeahorseContext*    seahorse_context_new                (gboolean           app_ctx,
+SeahorseContext*    seahorse_context_new                (guint              flags,
                                                          guint              ktype);
 
 void                seahorse_context_destroy            (SeahorseContext    *sctx);
 
+#define             seahorse_context_is_daemon(ctx)     ((ctx)->is_daemon)
+
 void                seahorse_context_add_key_source     (SeahorseContext    *sctx,
                                                          SeahorseKeySource  *sksrc);
 
@@ -122,15 +132,14 @@ guint               seahorse_context_get_count          (SeahorseContext    *sct
 
 SeahorseKey*        seahorse_context_get_key            (SeahorseContext    *sctx,
                                                          SeahorseKeySource  *sksrc,
-                                                         const gchar        *keyid);
+                                                         GQuark             keyid);
 
 GList*              seahorse_context_get_keys           (SeahorseContext    *sctx, 
                                                          SeahorseKeySource  *sksrc);
 
 SeahorseKey*        seahorse_context_find_key           (SeahorseContext    *sctx,
-                                                         GQuark             ktype,
-                                                         SeahorseKeyLoc     location,
-                                                         const gchar        *keyid);
+                                                         GQuark             keyid,
+                                                         SeahorseKeyLoc     location);
 
 GList*              seahorse_context_find_keys          (SeahorseContext    *sctx,
                                                          GQuark             ktype,
@@ -155,4 +164,30 @@ SeahorseOperation*  seahorse_context_load_remote_key    (SeahorseContext    *sct
 SeahorseOperation*  seahorse_context_load_remote_keys   (SeahorseContext    *sctx,
                                                          const gchar        *search);
 
+SeahorseOperation*  seahorse_context_transfer_keys      (SeahorseContext    *sctx, 
+                                                         GList              *keys, 
+                                                         SeahorseKeySource  *to,
+                                                         gboolean           individually);
+
+SeahorseOperation*  seahorse_context_retrieve_keys      (SeahorseContext    *sctx, 
+                                                         GQuark             ktype, 
+                                                         GSList             *keyids, 
+                                                         SeahorseKeySource  *to);
+
+GList*              seahorse_context_discover_keys      (SeahorseContext    *sctx, 
+                                                         GQuark             ktype, 
+                                                         GSList             *keyids);
+
+SeahorseKey*        seahorse_context_key_from_dbus      (SeahorseContext    *sctx,
+                                                         const gchar        *dbusid,
+                                                         guint              *uid);
+
+gchar*              seahorse_context_key_to_dbus        (SeahorseContext    *sctx,
+                                                         SeahorseKey        *skey,
+                                                         guint              uid);
+
+gchar*              seahorse_context_keyid_to_dbus      (SeahorseContext    *sctx,
+                                                         GQuark             keyid, 
+                                                         guint              uid);
+
 #endif /* __SEAHORSE_CONTEXT_H__ */
