@@ -19,14 +19,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <gnome.h>
 
 #include "seahorse-key-store.h"
+#include "seahorse-pgp-key.h"
 #include "seahorse-gconf.h"
+#include "seahorse-gtkstock.h"
+
+#ifdef WITH_SSH
+#include "seahorse-ssh-key.h"
+#endif 
 
 enum {
 	PROP_0,
-	PROP_KEY_SOURCE,
+	PROP_KEYSET,
     PROP_MODE,
     PROP_FILTER
 };
@@ -40,8 +47,6 @@ static const GType col_types[] = {
 };
 
 struct _SeahorseKeyStorePriv {    
-    GHashTable              *rows;
-    
     GtkTreeModelFilter      *filter;
     GtkTreeModelSort        *sort;
     
@@ -50,20 +55,12 @@ struct _SeahorseKeyStorePriv {
     guint                   filter_stag;
 };
 
-/* Internal data stored at 0 in the tree store in order to keep track
- * of the location, key-store and key.
- */
-typedef struct {
-	SeahorseKeyStore	*skstore;
-    GPtrArray           *refs;     /* GtkTreeRowReference pointers */
-	SeahorseKey		    *skey;     /* The key we're dealing with */
-} SeahorseKeyRow;                   
+G_DEFINE_TYPE (SeahorseKeyStore, seahorse_key_store, SEAHORSE_TYPE_KEY_MODEL);
 
-static void	seahorse_key_store_class_init		(SeahorseKeyStoreClass	*klass);
 static GObject* seahorse_key_store_constructor  (GType type, guint n_props, 
-                                             GObjectConstructParam* props);
+                                                 GObjectConstructParam* props);
 static void seahorse_key_store_dispose          (GObject       *gobject);
-static void	seahorse_key_store_finalize		    (GObject       *gobject);
+static void	seahorse_key_store_finalize         (GObject       *gobject);
 
 static void	seahorse_key_store_set_property		(GObject		*gobject,
 							 guint			prop_id,
@@ -89,115 +86,79 @@ static void	     seahorse_key_store_changed (SeahorseKeyStore   *skstore,
                                              SeahorseKeyChange  change);
 
 /* Key source signals */
-static void seahorse_key_store_key_added    (SeahorseKeySource  *sksrc,
+static void seahorse_key_store_key_added    (SeahorseKeyset     *skset,
                                              SeahorseKey        *skey,
                                              SeahorseKeyStore   *skstore);
-static void seahorse_key_store_key_removed  (SeahorseKeySource  *sksrc,
+static void seahorse_key_store_key_removed  (SeahorseKeyset     *skset,
                                              SeahorseKey        *skey,
+                                             gpointer           closure,
+                                             SeahorseKeyStore   *skstore);
+static void seahorse_key_store_key_changed  (SeahorseKeyset     *skset,
+                                             SeahorseKey        *skey,
+                                             SeahorseKeyChange  change,
+                                             gpointer           closure,
                                              SeahorseKeyStore   *skstore);
-                             
-/* Key signals */
-static void	seahorse_key_store_key_changed		(SeahorseKey		*skey,
-							 SeahorseKeyChange	change,
-							 SeahorseKeyRow		*skrow);
-/* Key Row methods */
-static void	seahorse_key_row_add			(SeahorseKeyStore	*skstore,
-                                             GtkTreeIter        *iter,
-                                             SeahorseKey        *skey);
-static void seahorse_key_row_remove         (SeahorseKeyRow     *skrow,
-                                             GtkTreeIter        *iter);
-static void seahorse_key_row_remove_all     (SeahorseKeyRow     *skrow);
-
-static void seahorse_key_row_free           (SeahorseKeyRow     *skrow);
 
 /* Filter row method */
 static gboolean filter_callback             (GtkTreeModel *model,
                                              GtkTreeIter *iter,
                                              gpointer data);
-                                             
-static GtkTreeStoreClass	*parent_class	= NULL;
-
-GType
-seahorse_key_store_get_type (void)
-{
-	static GType key_store_type = 0;
-	
-	if (!key_store_type) {
-		static const GTypeInfo key_store_info =
-		{
-			sizeof (SeahorseKeyStoreClass),
-			NULL, NULL,
-			(GClassInitFunc) seahorse_key_store_class_init,
-			NULL, NULL,
-			sizeof (SeahorseKeyStore),
-			0, NULL
-		};
-		
-		key_store_type = g_type_register_static (GTK_TYPE_TREE_STORE,
-			"SeahorseKeyStore", &key_store_info, 0);
-	}
-	
-	return key_store_type;
-}
 
 static void
 seahorse_key_store_class_init (SeahorseKeyStoreClass *klass)
 {
-	GObjectClass *gobject_class;
-	
-	parent_class = g_type_class_peek_parent (klass);
-	gobject_class = G_OBJECT_CLASS (klass);
-	
+    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+    
+    seahorse_key_store_parent_class = g_type_class_peek_parent (klass);
+
     gobject_class->constructor = seahorse_key_store_constructor;
     gobject_class->finalize = seahorse_key_store_finalize;
     gobject_class->dispose = seahorse_key_store_dispose;
-	gobject_class->set_property = seahorse_key_store_set_property;
-	gobject_class->get_property = seahorse_key_store_get_property;
-	
-	klass->append = seahorse_key_store_append;
-	klass->set = seahorse_key_store_set;
-	klass->changed = seahorse_key_store_changed;
+    gobject_class->set_property = seahorse_key_store_set_property;
+    gobject_class->get_property = seahorse_key_store_get_property;
+    
+    klass->append = seahorse_key_store_append;
+    klass->set = seahorse_key_store_set;
+    klass->changed = seahorse_key_store_changed;
   
-  	/* Class defaults. Derived classes should override */
-    klass->use_check = FALSE;
+    /* Class defaults. Derived classes should override */
     klass->use_icon = FALSE;
     klass->n_columns = KEY_STORE_NCOLS;
     klass->col_types = col_types;
     klass->col_ids = col_ids;
-	
-	g_object_class_install_property (gobject_class, PROP_KEY_SOURCE,
-		g_param_spec_object ("key-source", "Seahorse Key Source",
-				     "Current Seahorse Key Source to use",
-				     SEAHORSE_TYPE_KEY_SOURCE, G_PARAM_READWRITE));
-                    
+    
+    g_object_class_install_property (gobject_class, PROP_KEYSET,
+        g_param_spec_object ("keyset", "Seahorse Keyset", "Current Seahorse Key Source to use",
+                             SEAHORSE_TYPE_KEYSET, G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class, PROP_MODE,
-        g_param_spec_uint ("mode", "Key Store Mode",
-                     "Key store mode controls which keys to display",
-                     0, KEY_STORE_MODE_FILTERED, KEY_STORE_MODE_ALL, 
-                     G_PARAM_READWRITE));
+        g_param_spec_uint ("mode", "Key Store Mode", "Key store mode controls which keys to display",
+                           0, KEY_STORE_MODE_FILTERED, KEY_STORE_MODE_ALL, G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_FILTER,
-        g_param_spec_string ("filter", "Key Store Filter",
-                     "Key store filter for when in filtered mode",
-                     "", G_PARAM_READWRITE));
+        g_param_spec_string ("filter", "Key Store Filter", "Key store filter for when in filtered mode",
+                             "", G_PARAM_READWRITE));
+}
 
+static void
+seahorse_key_store_init (SeahorseKeyStore *skstore)
+{
+    
 }
 
 static GObject*  
 seahorse_key_store_constructor (GType type, guint n_props, GObjectConstructParam* props)
 {
-    GObject* obj = G_OBJECT_CLASS (parent_class)->constructor (type, n_props, props);
+    GObject* obj = G_OBJECT_CLASS (seahorse_key_store_parent_class)->constructor (type, n_props, props);
     SeahorseKeyStore* skstore = SEAHORSE_KEY_STORE (obj);
 
     /* init private vars */
     skstore->priv = g_new0 (SeahorseKeyStorePriv, 1);
-    skstore->priv->rows = g_hash_table_new_full (g_direct_hash, g_direct_equal, 
-                                       NULL, (GDestroyNotify)seahorse_key_row_free);
  
     /* Setup the store */
     guint cols = SEAHORSE_KEY_STORE_GET_CLASS (skstore)->n_columns;
     GType* types = (GType*)SEAHORSE_KEY_STORE_GET_CLASS (skstore)->col_types;
-    gtk_tree_store_set_column_types (GTK_TREE_STORE (obj), cols, types);
+    seahorse_key_model_set_column_types (SEAHORSE_KEY_MODEL (obj), cols, types);
     
     /* Setup the sort and filter */
     skstore->priv->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (obj), NULL));
@@ -223,44 +184,42 @@ seahorse_key_store_dispose (GObject *gobject)
 
     skstore = SEAHORSE_KEY_STORE (gobject);  
 
-    if (skstore->sksrc) {
-        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+    if (skstore->skset) {
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
                                 seahorse_key_store_key_added, skstore);
-        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
                                 seahorse_key_store_key_removed, skstore);
-        g_object_unref (skstore->sksrc);        
-        skstore->sksrc = NULL;
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
+                                seahorse_key_store_key_changed, skstore);
+        g_object_unref (skstore->skset);        
+        skstore->skset = NULL;
     }
     
-    G_OBJECT_CLASS (parent_class)->dispose (gobject);
+    G_OBJECT_CLASS (seahorse_key_store_parent_class)->dispose (gobject);
 }
     
 static void
 seahorse_key_store_finalize (GObject *gobject)
 {
-	SeahorseKeyStore *skstore;
-	
-	skstore = SEAHORSE_KEY_STORE (gobject);
-    g_assert (skstore->sksrc == NULL);
-    	
+    SeahorseKeyStore *skstore = SEAHORSE_KEY_STORE (gobject);
+    g_assert (skstore->skset == NULL);
+    
     /* These were allocated in the constructor */
     g_object_unref (skstore->priv->sort);
     g_object_unref (skstore->priv->filter);
      
     /* Allocated in property setter */
     g_free (skstore->priv->filter_text); 
-	
-    /* The row cache */
-    g_hash_table_destroy (skstore->priv->rows);
     
-	G_OBJECT_CLASS (parent_class)->finalize (gobject);
+    G_OBJECT_CLASS (seahorse_key_store_parent_class)->finalize (gobject);
 }
 
 /* Refilter the tree */
 static gboolean
 refilter_now(SeahorseKeyStore* skstore)
 {
-    gtk_tree_model_filter_refilter (skstore->priv->filter);
+    seahorse_keyset_refresh (skstore->skset);
+    gtk_tree_model_filter_refilter (skstore->priv->filter);    
     skstore->priv->filter_stag = 0;
     return FALSE;
 }
@@ -284,15 +243,16 @@ seahorse_key_store_set_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		/* Connects to key source signals */
-		case PROP_KEY_SOURCE:
-            g_return_if_fail (skstore->sksrc == NULL);
-			skstore->sksrc = g_value_get_object (value);
-			g_object_ref (skstore->sksrc);
-            g_signal_connect_after (skstore->sksrc, "added",
+        case PROP_KEYSET:
+            g_assert (skstore->skset == NULL);
+            skstore->skset = g_value_get_object (value);
+            g_object_ref (skstore->skset);
+            g_signal_connect_after (skstore->skset, "added",
                 G_CALLBACK (seahorse_key_store_key_added), skstore);
-            g_signal_connect_after (skstore->sksrc, "removed",
+            g_signal_connect_after (skstore->skset, "removed",
                 G_CALLBACK (seahorse_key_store_key_removed), skstore);
+            g_signal_connect_after (skstore->skset, "changed",
+                G_CALLBACK (seahorse_key_store_key_changed), skstore);
 			break;
 			
 		/* The filtering mode */
@@ -335,8 +295,8 @@ seahorse_key_store_get_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		case PROP_KEY_SOURCE:
-			g_value_set_object (value, skstore->sksrc);
+		case PROP_KEYSET:
+			g_value_set_object (value, skstore->skset);
 			break;
 			
 		/* The filtering mode */
@@ -361,8 +321,8 @@ static gboolean
 seahorse_key_store_append (SeahorseKeyStore *skstore, SeahorseKey *skey, 
                            guint uid, GtkTreeIter *iter)
 {
-	SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, uid, iter);
-	seahorse_key_row_add (skstore, iter, skey);
+    SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, uid, iter);
+    seahorse_key_model_set_row_key (SEAHORSE_KEY_MODEL (skstore), iter, skey);
     return FALSE;
 }
 
@@ -371,14 +331,15 @@ static void
 seahorse_key_store_set (SeahorseKeyStore *skstore, SeahorseKey *skey, 
                         guint uid, GtkTreeIter *iter)
 {
-    gchar *userid = seahorse_key_get_userid (skey, uid);
-    gboolean pair = SEAHORSE_IS_KEY_PAIR(skey);
-	gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
-        KEY_STORE_CHECK, FALSE,
-        KEY_STORE_PAIR, uid == 0 ? pair : FALSE,
-        KEY_STORE_NOTPAIR, uid == 0 ? !pair : FALSE,
-		KEY_STORE_NAME, userid,
-		KEY_STORE_KEYID, seahorse_key_get_keyid (skey, 0),
+    gchar *userid = seahorse_key_get_name (skey, uid);
+    gboolean sec = seahorse_key_get_etype (skey) == SKEY_PRIVATE;
+    const gchar *stockid = seahorse_key_get_stock_id (skey);
+
+    gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
+        KEY_STORE_PAIR, uid == 0 ? sec : FALSE,
+        KEY_STORE_STOCK_ID, uid == 0 ? stockid : NULL,
+        KEY_STORE_NAME, userid,
+        KEY_STORE_KEYID, seahorse_key_get_short_keyid (skey),
         KEY_STORE_UID, uid, -1);
     g_free (userid);
 }
@@ -400,10 +361,10 @@ seahorse_key_store_changed (SeahorseKeyStore *skstore, SeahorseKey *skey, guint
 
 /* Appends @skey and all uids */
 static void
-seahorse_key_store_key_added (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_added (SeahorseKeyset *skset, SeahorseKey *skey, SeahorseKeyStore *skstore)
 {
-	GtkTreeIter iter;
-	guint i, uids = seahorse_key_get_num_uids (skey);
+	GtkTreeIter iter = { 0, };
+	guint i, uids = seahorse_key_get_num_names (skey);
  
     for (i = 0; i < uids; i++) {
     	if(!SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, i, &iter))
@@ -411,47 +372,45 @@ seahorse_key_store_key_added (SeahorseKeySource *sksrc, SeahorseKey *skey, Seaho
     }
 }
 
-/* Removes @skrow */
+/* Removes all rows for key */
 static void
-seahorse_key_store_key_removed (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_removed (SeahorseKeyset *skset, SeahorseKey *skey, 
+                                gpointer closure, SeahorseKeyStore *skstore)
 {
-    SeahorseKeyRow *skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
-    if (skrow)
-	    seahorse_key_row_remove_all (skrow);
+    seahorse_key_model_remove_rows_for_key (SEAHORSE_KEY_MODEL (skstore), skey);
 }
 
 /* Calls virtual |changed| for all relevant uids. adds new uids if necessary */
 static void
-seahorse_key_store_key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseKeyRow *skrow)
+seahorse_key_store_key_changed (SeahorseKeyset *skset, SeahorseKey *skey, 
+                                SeahorseKeyChange change, gpointer closure, SeahorseKeyStore *skstore)
 {
     guint i, uid, old_uids, num_uids;
     GtkTreeIter first;
-    GtkTreeIter iter;
-    GtkTreePath *path;
+    GtkTreeIter *iter;
+    GSList *rows, *l;
     
     old_uids = 0;
-    num_uids = seahorse_key_get_num_uids (skey);
+    num_uids = seahorse_key_get_num_names (skey);
+    rows = seahorse_key_model_get_rows_for_key (SEAHORSE_KEY_MODEL (skstore), skey);
     
-    for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
-        path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
-        g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path));
-        gtk_tree_path_free (path);        
-
-        gtk_tree_model_get (GTK_TREE_MODEL (skrow->skstore), &iter, KEY_STORE_UID, &uid, -1);
+    for (l = rows; l; l = g_slist_next (l)) {
+        
+        iter = (GtkTreeIter*)l->data;
+        gtk_tree_model_get (GTK_TREE_MODEL (skstore), iter, KEY_STORE_UID, &uid, -1);
         
         /* Remove any extra rows */
         if (uid >= num_uids) {
-            seahorse_key_row_remove (skrow, &iter);
-            i--;
+            seahorse_key_model_set_row_key (SEAHORSE_KEY_MODEL (skstore), iter, NULL);
+            gtk_tree_store_remove (GTK_TREE_STORE (skstore), iter);
             continue;
         }
 
-        SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->changed (skrow->skstore, skey, uid, &iter, change);
+        SEAHORSE_KEY_STORE_GET_CLASS (skstore)->changed (skstore, skey, uid, iter, change);
 
         /* The top parent row */
         if (uid == 0)
-            memcpy (&first, &iter, sizeof (first));            
+            memcpy (&first, iter, sizeof (first));            
 
         /* Find the max uid on the rows */
         if (uid >= old_uids)
@@ -460,7 +419,9 @@ seahorse_key_store_key_changed (SeahorseKey *skey, SeahorseKeyChange change, Sea
 
     /* Add all new rows */    
     for (i = old_uids; i < num_uids; i++)
-        SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->append (skrow->skstore, skey, i, &first);
+        SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, i, &first);
+    
+    seahorse_key_model_free_rows (rows);
 }
 
 /* Update the sort order for a column */
@@ -538,12 +499,11 @@ sort_changed(GtkTreeSortable *sort, gpointer user_data)
 static SeahorseKeyStore* 
 key_store_from_model (GtkTreeModel *model)
 {
-	/* Sort models are what's set on the tree */
-    if (GTK_IS_TREE_MODEL_SORT (model)) {
+    /* Sort models are what's set on the tree */
+    if (GTK_IS_TREE_MODEL_SORT (model))
         model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+    if (GTK_IS_TREE_MODEL_FILTER (model)) 
         model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-    }
-    
     if (SEAHORSE_IS_KEY_STORE (model))
         return SEAHORSE_KEY_STORE (model);
     
@@ -551,169 +511,6 @@ key_store_from_model (GtkTreeModel *model)
     return NULL;
 }
 
-/* Called when a checkbox is toggled, toggle the value */
-static void
-check_toggled(GtkCellRendererToggle *cellrenderertoggle, gchar *path, gpointer user_data)
-{
-    GtkTreeView *view = GTK_TREE_VIEW (user_data);
-    SeahorseKeyStore *skstore = key_store_from_model (gtk_tree_view_get_model (view));
-    GtkTreeModel* fmodel = GTK_TREE_MODEL (skstore->priv->sort);
-    GtkTreeSelection *selection;
-    gboolean prev = FALSE;
-    GtkTreeIter iter;
-    GtkTreeIter child;
-    GValue v;
-
-    memset (&v, 0, sizeof(v));
-    g_return_if_fail (path != NULL);
-    g_return_if_fail (gtk_tree_model_get_iter_from_string (fmodel, &iter, path));
-    
-    /* We get notified in filtered coordinates, we have to convert those to base */
-    seahorse_key_store_get_base_iter (skstore, &child, &iter);
- 
-    gtk_tree_model_get_value (GTK_TREE_MODEL (skstore), &child, KEY_STORE_CHECK, &v);
-    if(G_VALUE_TYPE (&v) == G_TYPE_BOOLEAN)
-        prev = g_value_get_boolean (&v);
-    g_value_unset (&v);    
-    
-    gtk_tree_store_set (GTK_TREE_STORE (skstore), &child, KEY_STORE_CHECK, prev ? FALSE : TRUE, -1);
-
-    selection = gtk_tree_view_get_selection (view);
-    g_signal_emit_by_name (selection, "changed");    
-}
-
-static void
-row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2, 
-               SeahorseKeyStore *skstore)
-{
-    GtkTreeModel* fmodel = GTK_TREE_MODEL (skstore->priv->sort);
-    GtkTreeSelection *selection;
-    gboolean prev = FALSE;
-    GtkTreeIter iter;
-    GtkTreeIter child;
-    GValue v;
-
-    memset (&v, 0, sizeof(v));
-    g_return_if_fail (path != NULL);
-    g_return_if_fail (gtk_tree_model_get_iter (fmodel, &iter, path));
-    
-    /* We get notified in filtered coordinates, we have to convert those to base */
-    seahorse_key_store_get_base_iter (skstore, &child, &iter);
- 
-    gtk_tree_model_get_value (GTK_TREE_MODEL (skstore), &child, KEY_STORE_CHECK, &v);
-    if(G_VALUE_TYPE (&v) == G_TYPE_BOOLEAN)
-        prev = g_value_get_boolean (&v);
-    g_value_unset (&v);    
-    
-    gtk_tree_store_set (GTK_TREE_STORE (skstore), &child, KEY_STORE_CHECK, prev ? FALSE : TRUE, -1);
-
-    selection = gtk_tree_view_get_selection (treeview);
-    g_signal_emit_by_name (selection, "changed");    
-}
-
-/* Creates a new #SeahorseKeyRow for listening to key signals */
-static void
-seahorse_key_row_add (SeahorseKeyStore *skstore, GtkTreeIter *iter, SeahorseKey *skey)
-{
-	SeahorseKeyRow *skrow;
-	GtkTreePath *path;
-    
-    /* Do we already have a row for this key? */
-    skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
-    
-    if (!skrow) {
-        skrow = g_new0 (SeahorseKeyRow, 1);
-        skrow->refs = g_ptr_array_new ();
-        skrow->skstore = skstore;
-
-        skrow->skey = skey;
-        g_object_ref (skey);
-
-        g_signal_connect_after (skrow->skey, "changed",
-                                G_CALLBACK (seahorse_key_store_key_changed), skrow);
-
-        /* Put it in our row cache */
-        g_hash_table_replace (skstore->priv->rows, skey, skrow);
-    }
-    
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (skstore), iter);
-    g_ptr_array_add (skrow->refs, gtk_tree_row_reference_new (GTK_TREE_MODEL (skstore), path));
-    gtk_tree_path_free (path);
-    
-    gtk_tree_store_set (GTK_TREE_STORE (skstore), iter, KEY_STORE_DATA, skrow, -1);
-}
-
-static void
-seahorse_key_row_free (SeahorseKeyRow *skrow)
-{
-    guint i;
-    
-    /* Unref key */
-    g_signal_handlers_disconnect_by_func (skrow->skey,
-                                          seahorse_key_store_key_changed, skrow);
-    g_object_unref (skrow->skey);
-  
-    for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
-        gtk_tree_row_reference_free ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
-    }
-    g_ptr_array_free (skrow->refs, TRUE);
-    
-    g_free (skrow);
-}    
-
-/* Calls virtual remove() for @skrow's location, disconnects and unrefs
- * key, then frees itself.
- */
-static void
-seahorse_key_row_remove_all (SeahorseKeyRow *skrow)
-{
-	GtkTreeIter iter;
-	GtkTreePath *path;
-    guint i;
-    
-    for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
-        path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
-        
-        /* Note that removing a parent row could remove it's sub rows, so ... */
-        if (path) {
-            if(gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path))
-                gtk_tree_store_remove (GTK_TREE_STORE (skrow->skstore), &iter);
-            gtk_tree_path_free (path);
-        }
-    }
-
-    /* This also frees the skrow */
-    g_return_if_fail (g_hash_table_remove (skrow->skstore->priv->rows, skrow->skey));
-}
-
-static void
-seahorse_key_row_remove (SeahorseKeyRow *skrow, GtkTreeIter *iter)
-{
-    GtkTreePath *p1, *p2;
-    guint i, r;
-    
-    p1 = gtk_tree_model_get_path (GTK_TREE_MODEL (skrow->skstore), iter);
-    
-    for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
-        p2 = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
-        r = gtk_tree_path_compare (p1, p2);
-        gtk_tree_path_free (p2);
-        
-        if (r == 0) {
-            g_ptr_array_remove_index (skrow->refs, i);
-            gtk_tree_store_remove (GTK_TREE_STORE (skrow->skstore), iter);
-            break;
-        }
-    }
-    
-    /* This also frees the skrow */
-    if (skrow->refs->len == 0)
-        g_return_if_fail (g_hash_table_remove (skrow->skstore->priv->rows, skrow->skey));
-}
-
 /**
  * seahorse_key_store_init:
  * @skstore: #SeahorseKeyStore to initialize
@@ -723,7 +520,7 @@ seahorse_key_row_remove (SeahorseKeyRow *skrow, GtkTreeIter *iter)
  * This must be called after creating a new #SeahorseKeyStore.
  **/
 void
-seahorse_key_store_init (SeahorseKeyStore *skstore, GtkTreeView *view)
+seahorse_key_store_initialize (SeahorseKeyStore *skstore, GtkTreeView *view)
 {
 	GtkTreeViewColumn *col;
 
@@ -732,56 +529,26 @@ seahorse_key_store_init (SeahorseKeyStore *skstore, GtkTreeView *view)
     /* The sorted model is the top level model */   
     g_assert (GTK_IS_TREE_MODEL (skstore->priv->sort));
     gtk_tree_view_set_model (view, GTK_TREE_MODEL (skstore->priv->sort));
- 
- 	/* When using checks we add a check column */
-    if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
-        GtkCellRenderer *renderer;
-        renderer = gtk_cell_renderer_toggle_new ();
-        g_signal_connect (renderer, "toggled", G_CALLBACK (check_toggled), view);
-        col = gtk_tree_view_column_new_with_attributes ("", renderer, "active", KEY_STORE_CHECK, NULL);
-        gtk_tree_view_column_set_resizable (col, FALSE);
-        gtk_tree_view_append_column (view, col);
-        g_signal_connect (view, "row_activated", G_CALLBACK (row_activated), skstore);
-    }
-    
+
     /* When using key pair icons, we add an icon column */
     if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_icon) {
-    	GtkCellRenderer *secretPixbufRenderer = NULL, *keyPixbufRenderer = NULL;
-    	GdkPixbuf *secret_pixbuf = NULL, *key_pixbuf;
-    	
-    	secret_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-secret.png", NULL);
-    	secretPixbufRenderer = gtk_cell_renderer_pixbuf_new();
-    	g_object_set(secretPixbufRenderer,
-    				 "pixbuf", GDK_PIXBUF(secret_pixbuf),
-    				 NULL);
-    	
-    	key_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-key.png", NULL);
-    	keyPixbufRenderer = gtk_cell_renderer_pixbuf_new();
-    	g_object_set(keyPixbufRenderer,
-    				 "pixbuf", GDK_PIXBUF(key_pixbuf),
-    				 NULL);
-    				 			 
-    	col = gtk_tree_view_column_new_with_attributes ("", secretPixbufRenderer, 
-   												"visible", KEY_STORE_PAIR,
+    	GtkCellRenderer  *renderer = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (renderer, "stock-size", GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
+    	col = gtk_tree_view_column_new_with_attributes ("", renderer, 
+   												"stock-id", KEY_STORE_STOCK_ID,
    												NULL);
-
-   		gtk_tree_view_column_pack_end(col,
-   									  keyPixbufRenderer,
-   									  FALSE);
-   									  
-   		gtk_tree_view_column_add_attribute(col, keyPixbufRenderer,
-   												"visible",
-   												KEY_STORE_NOTPAIR); 										
+										
         gtk_tree_view_column_set_resizable (col, FALSE);
         gtk_tree_view_append_column (view, col);
         
         gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_PAIR);
-      }
-      
-	col = seahorse_key_store_append_column (view, _("Name"), KEY_STORE_NAME);
-	gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_NAME);
-	
-	seahorse_key_store_append_column (view, _("Key ID"), KEY_STORE_KEYID);
+    }
+    
+    col = seahorse_key_store_append_column (view, _("Name"), KEY_STORE_NAME);
+    gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_NAME);
+    
+    col = seahorse_key_store_append_column (view, _("Key ID"), KEY_STORE_KEYID);
+    gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_KEYID);
 
     if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->gconf_sort_key) {
         /* Also watch for sort-changed on the store */
@@ -801,22 +568,18 @@ seahorse_key_store_populate (SeahorseKeyStore *skstore)
 {
 	GList *keys, *list = NULL;
 	SeahorseKey *skey;
-	guint count = 1;
-	gdouble length;
 	
     g_return_if_fail (SEAHORSE_IS_KEY_STORE (skstore));
-    g_return_if_fail (SEAHORSE_IS_KEY_SOURCE (skstore->sksrc));
+    g_return_if_fail (SEAHORSE_IS_KEYSET (skstore->skset));
 	
     /* Don't precipitate a load */
-    if (seahorse_key_source_get_count (skstore->sksrc, FALSE) > 0) {
+    if (seahorse_keyset_get_count (skstore->skset) > 0) {
 
-    	keys = list = seahorse_key_source_get_keys (skstore->sksrc, FALSE);
-    	length = g_list_length (list);
+    	keys = list = seahorse_keyset_get_keys (skstore->skset);
 	
     	while (list != NULL && (skey = list->data) != NULL) {
-            seahorse_key_store_key_added (skstore->sksrc, skey, skstore);
+            seahorse_key_store_key_added (skstore->skset, skey, skstore);
     		list = g_list_next (list);
-    		count++;
     	}
      
         g_list_free (keys);
@@ -828,8 +591,7 @@ static SeahorseKey*
 key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter, guint *uid)
 {
     GtkTreeIter i;
-    SeahorseKeyRow *skrow;
-    SeahorseKey *return_key = NULL;
+    SeahorseKey *skey = NULL;
     
     /* Convert to base iter if necessary */
     if (!SEAHORSE_IS_KEY_STORE (model)) {
@@ -840,11 +602,10 @@ key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter, guint *uid)
         model = GTK_TREE_MODEL (skstore);
     }
     
-    gtk_tree_model_get (model, iter, 
-        KEY_STORE_DATA, &skrow, uid ? KEY_STORE_UID : -1, uid, -1);    
-    if (skrow)
-        return_key = skrow->skey;
-    return return_key;
+    skey = seahorse_key_model_get_row_key (SEAHORSE_KEY_MODEL (model), iter);
+    if (skey && uid)
+        gtk_tree_model_get (model, iter, KEY_STORE_UID, uid, -1);
+    return skey;
 }
 
 
@@ -927,7 +688,7 @@ seahorse_key_store_get_all_keys (GtkTreeView *view)
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     skstore = key_store_from_model (gtk_tree_view_get_model (view));
     
-    return seahorse_key_source_get_keys (skstore->sksrc, FALSE);
+    return seahorse_keyset_get_keys (skstore->skset);
 }
 
 /**
@@ -941,43 +702,28 @@ seahorse_key_store_get_all_keys (GtkTreeView *view)
 GList*
 seahorse_key_store_get_selected_keys (GtkTreeView *view)
 {
-	GList *l, *keys = NULL;
+    SeahorseKey *skey;
+    GList *l, *keys = NULL;
     SeahorseKeyStore* skstore;
+    GList *list, *paths = NULL;
+    GtkTreeSelection *selection;    
     
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     skstore = key_store_from_model (gtk_tree_view_get_model (view));
     
-    if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
-        GtkTreeModel* model = GTK_TREE_MODEL (skstore);
-        GtkTreeIter iter;
-        gboolean check;
-            
-        if (gtk_tree_model_get_iter_first (model, &iter)) {
-            do {
-                check = FALSE;
-                gtk_tree_model_get (model, &iter, KEY_STORE_CHECK, &check, -1); 
-                if (check)
-                    keys = g_list_append (keys, key_from_iterator (model, &iter, NULL));
-            } while (gtk_tree_model_iter_next (model, &iter));
-        }
+    selection = gtk_tree_view_get_selection (view);
+    paths = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+    /* make key list */
+    for (list = paths; list != NULL; list = g_list_next (list)) {
+        skey = seahorse_key_store_get_key_from_path (view, list->data, NULL);
+        if (skey != NULL)
+            keys = g_list_append (keys, skey);
     }
-    
-    /* Fall back if none checked, or not using checks */
-    if (keys == NULL) {
-    	GList *list, *paths = NULL;
-        GtkTreeSelection *selection;	
         
-    	selection = gtk_tree_view_get_selection (view);
-    	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
-	
-    	/* make key list */
-	    for (list = paths; list != NULL; list = g_list_next (list))
-		    keys = g_list_append (keys, seahorse_key_store_get_key_from_path (view, list->data, NULL));
-            
-        /* free selected paths */
-        g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
-        g_list_free (paths);
-    }
+    /* free selected paths */
+    g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
+    g_list_free (paths);
     
     /* Remove duplicates */
     keys = g_list_sort (keys, compare_pointers);
@@ -985,10 +731,42 @@ seahorse_key_store_get_selected_keys (GtkTreeView *view)
         while (l->next && l->data == l->next->data)
             keys = g_list_delete_link (keys, l->next);
     }    
-	 
+    
     return keys;
 }
 
+void
+seahorse_key_store_set_selected_keys (GtkTreeView *view, GList* keys)
+{
+    SeahorseKeyStore* skstore;
+    GtkTreeSelection* selection;
+    GList *l;
+    GSList *rows, *rl;
+    GtkTreeIter it;
+    
+    g_return_if_fail (GTK_IS_TREE_VIEW (view));
+    selection = gtk_tree_view_get_selection (view);
+    gtk_tree_selection_unselect_all (selection);
+    
+    skstore = key_store_from_model (gtk_tree_view_get_model (view));
+    g_return_if_fail (SEAHORSE_IS_KEY_STORE (skstore));
+    
+    for(l = keys; l; l = g_list_next (l)) {
+        
+        /* Get all the rows for that key .... */
+        rows = seahorse_key_model_get_rows_for_key (SEAHORSE_KEY_MODEL (skstore), 
+                                                    SEAHORSE_KEY (l->data));
+        for(rl = rows; rl; rl = g_slist_next (rl)) {
+
+            /* And select them ... */
+            if(seahorse_key_store_get_upper_iter(skstore, &it, (GtkTreeIter*)rl->data))
+                gtk_tree_selection_select_iter (selection, &it);
+        }
+        
+        seahorse_key_model_free_rows (rows);
+    }
+}
+
 /**
  * seahorse_key_store_get_selected_key:
  * @view: #GtkTreeView with selection
@@ -1003,46 +781,26 @@ SeahorseKey*
 seahorse_key_store_get_selected_key (GtkTreeView *view, guint *uid)
 {
     SeahorseKeyStore* skstore;
-	SeahorseKey *skey = NULL;
-	
+    SeahorseKey *skey = NULL;
+    GList *paths = NULL;
+    GtkTreeSelection *selection;
+    
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     skstore = key_store_from_model (gtk_tree_view_get_model (view));
     
-    if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
-        GtkTreeModel* model = GTK_TREE_MODEL (skstore);
-        GtkTreeIter iter;
-        gboolean check;
-            
-        if (gtk_tree_model_get_iter_first (model, &iter)) {
-            do {
-                check = FALSE;
-                gtk_tree_model_get (model, &iter, KEY_STORE_CHECK, &check, -1); 
-                if (check) {
-					skey = key_from_iterator (model, &iter, uid);
-					break;
-				}
-            } while (gtk_tree_model_iter_next (model, &iter));
-        }
-    }
-
-    /* Fall back if none checked, or not using checks */
-    if (skey == NULL) {
-    	GList *paths = NULL;
-        GtkTreeSelection *selection;	
-        
-    	selection = gtk_tree_view_get_selection (view);
-    	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
-	
-    	/* make key list */
-		if (paths != NULL)
-		    skey = seahorse_key_store_get_key_from_path (view, paths->data, uid);
+    selection = gtk_tree_view_get_selection (view);
+    paths = gtk_tree_selection_get_selected_rows (selection, NULL);
+    
+    /* make key list */
+    if (paths != NULL) {
+        skey = seahorse_key_store_get_key_from_path (view, paths->data, uid);
             
         /* free selected paths */
         g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
         g_list_free (paths);
     }
-		
-	return skey;
+    
+    return skey;
 }
 
 /* Search through row for text */
@@ -1095,14 +853,6 @@ filter_callback (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
         ret = row_contains_filtered_text (model, iter, skstore->priv->filter_text);
         break;
         
-    case KEY_STORE_MODE_SELECTED:
-        if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
-            gboolean check = FALSE;
-            gtk_tree_model_get (model, iter, KEY_STORE_CHECK, &check, -1); 
-            ret = check;
-        }
-        break;
-        
     case KEY_STORE_MODE_ALL:
         ret = TRUE;
         break;
@@ -1124,8 +874,8 @@ filter_callback (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 
 /* Given a treeview iter, get the base store iterator */
 void 
-seahorse_key_store_get_base_iter(SeahorseKeyStore* skstore, GtkTreeIter* base_iter, 
-                                        const GtkTreeIter* iter)
+seahorse_key_store_get_base_iter(SeahorseKeyStore *skstore, GtkTreeIter *base_iter, 
+                                 const GtkTreeIter *iter)
 {
     GtkTreeIter i;
     
@@ -1135,3 +885,34 @@ seahorse_key_store_get_base_iter(SeahorseKeyStore* skstore, GtkTreeIter* base_it
     gtk_tree_model_sort_convert_iter_to_child_iter (skstore->priv->sort, &i, (GtkTreeIter*)iter);
     gtk_tree_model_filter_convert_iter_to_child_iter (skstore->priv->filter, base_iter, &i);
 }
+
+/* Given a base store iter, get the treeview iter */
+gboolean 
+seahorse_key_store_get_upper_iter(SeahorseKeyStore *skstore, GtkTreeIter *upper_iter, 
+                                  const GtkTreeIter *iter)
+{
+    GtkTreeIter i;
+    GtkTreePath *child_path, *path;
+    gboolean ret;
+    
+    g_return_val_if_fail (SEAHORSE_IS_KEY_STORE (skstore), FALSE);
+    g_assert (skstore->priv->sort && skstore->priv->filter);
+    
+    child_path = gtk_tree_model_get_path (gtk_tree_model_filter_get_model (skstore->priv->filter), 
+                                          (GtkTreeIter*)iter);
+    g_return_val_if_fail (child_path != NULL, FALSE);
+    path = gtk_tree_model_filter_convert_child_path_to_path (skstore->priv->filter, child_path);
+    gtk_tree_path_free (child_path);
+
+    if (!path)
+        return FALSE;
+
+    ret = gtk_tree_model_get_iter (GTK_TREE_MODEL (skstore->priv->filter), &i, path);
+    gtk_tree_path_free (path);
+    
+    if (!ret)
+        return FALSE;
+  
+    gtk_tree_model_sort_convert_child_iter_to_iter (skstore->priv->sort, upper_iter, &i);
+    return TRUE;
+}
